/* Prologue. */
%expect 0
%error-verbose
%debug
%defines
%locations

%code provides // *.h
{
# define yyerror(Msg)                           \
  yyerror_(&yylloc, Msg)
}

%code requires // *.c
{
#include <stdio.h>
#include <stdlib.h>
  int yylex();
  struct YYLTYPE;
  void loc_print(FILE* f, const struct YYLTYPE* l);
  void yyerror_(const struct YYLTYPE* loc, const char* msg);
}


%union
{
  int ival;
  float fval;
}

%printer { fprintf(stderr, "%d", $$); } <ival>;

%token STAR "*"
       SLASH "/"
       PLUS "+"
       MINUS "-"
       LPAREN "(" RPAREN ")"
       EOL "end of line"

%token<ival> NUMBER "number"
%type<ival> exp

%left "+" "-"
%left "*" "/"

%%
/* Rules. */

input:
  /* empty */
| line input
;

line:
        "end of line"
| exp   "end of line"   { printf("%d\n", $1); }
| error "end of line"
;

exp:
  exp "+" exp  { $$ = $1 + $3; }
| exp "-" exp  { $$ = $1 - $3; }
| exp "*" exp  { $$ = $1 * $3; }
| exp "/" exp
   {
     if ($3)
       $$ = $1 / $3;
     else
       yyerror_(&@$, "division by 0");
   }
| "number"     { $$ = $1; }
| "(" exp ")"  { $$ = $2; }
| "(" error ")" { $$ = 666; }
;

%%
/* Epilogue. */
unsigned mynerrs = 0;

void
loc_print(FILE* f, const YYLTYPE* l)
{
  fprintf (f, "%d.%d-%d.%d",
           l->first_line, l->first_column,
           l->last_line, l->last_column);
}

void
yyerror_(const YYLTYPE* loc, const char* msg)
{
  ++mynerrs;
  loc_print(stderr, loc);
  fprintf(stderr, ": %s\n", msg);
}

int
main()
{
  yydebug = !!getenv("YYDEBUG");
  yyparse();
  return !!mynerrs;
}
