/* -*- C -*- */
/* Prologue. */

%option nounput noyywrap

%{
#include "parse-calc.h"
char buf[BUFSIZ];

#define YYLLOC_STEP()                           \
 do {                                           \
   yylloc.first_line = yylloc.last_line;        \
   yylloc.first_column = yylloc.last_column;    \
 } while (0)

#define YYLLOC_EOL(Num)                         \
 do {                                           \
   yylloc.last_line += Num;                     \
   yylloc.last_column = 1;                      \
 } while (0)

#define YY_USER_ACTION                          \
  yylloc.last_column += yyleng;
%}

%%
%{
  YYLLOC_STEP();
%}

 /* Rules. */

[0-9]+      yylval.ival = strtol(yytext, 0, 10); return NUMBER;

"+"|"add"   return PLUS;
"-"|"sub"   return MINUS;
"*"|"mult"  return STAR;
"/"|"div"   return SLASH;

"("         return LPAREN;
")"         return RPAREN;

[\t ]+      YYLLOC_STEP();

\n+         YYLLOC_EOL(yyleng); return EOL;

.    {
  snprintf(buf, sizeof buf/sizeof buf[0],
           "unexpected character: `%c'", yytext[0]);
  yyerror(buf);
}
%%
/* Epilogue. */
